@@ -72,6 +72,31 @@ G_GNUC_INTERNAL void _gio_marshal_BOOLEAN__UINT (GClosure     *closure,
                                                  gpointer      marshal_data);
 #define _gio_marshal_BOOL__UINT	_gio_marshal_BOOLEAN__UINT
 
+/* VOID:STRING,STRING,BOXED (./gio-marshal.list:9) */
+G_GNUC_INTERNAL void _gio_marshal_VOID__STRING_STRING_BOXED (GClosure     *closure,
+                                                             GValue       *return_value,
+                                                             guint         n_param_values,
+                                                             const GValue *param_values,
+                                                             gpointer      invocation_hint,
+                                                             gpointer      marshal_data);
+
+/* VOID:BOOL,BOXED (./gio-marshal.list:10) */
+G_GNUC_INTERNAL void _gio_marshal_VOID__BOOLEAN_BOXED (GClosure     *closure,
+                                                       GValue       *return_value,
+                                                       guint         n_param_values,
+                                                       const GValue *param_values,
+                                                       gpointer      invocation_hint,
+                                                       gpointer      marshal_data);
+#define _gio_marshal_VOID__BOOL_BOXED	_gio_marshal_VOID__BOOLEAN_BOXED
+
+/* VOID:BOXED,BOXED (./gio-marshal.list:11) */
+G_GNUC_INTERNAL void _gio_marshal_VOID__BOXED_BOXED (GClosure     *closure,
+                                                     GValue       *return_value,
+                                                     guint         n_param_values,
+                                                     const GValue *param_values,
+                                                     gpointer      invocation_hint,
+                                                     gpointer      marshal_data);
+
 G_END_DECLS
 
 #endif /* ___gio_marshal_MARSHAL_H__ */
